using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.ServiceModel;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using VEIS.Plugins.Messages;
using System.Web.Script.Serialization;
using System.Runtime.Serialization;
using System.Activities;
using System.Text;
using VEIS.Plugins.Models;
using VEIS.Plugins;

namespace VEIS.Plugins
{
    public partial class MVISearchAction : BaseAction
    {
        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                Response.Set(context, "");
                // Use a 'using' statement to dispose of the service context properly
                // To use a specific early bound entity replace the 'Entity' below with the appropriate class type
                using (var localContext = new ActionContext(context))
                {
                    MVISearchRequest request = JsonHelper.Deserialize<MVISearchRequest>(this.Request.Get<string>(context));
                    if (request.Query == null)
                    {
                        throw new ArgumentNullException("Query Not Specified in Request");
                    }

                    PersonSearchRequest searchRequest = new PersonSearchRequest();
                    VeisConfig config = localContext.RetrieveVeisConfig("patsr_mviserviceurl", "patsr_mvisubscriptionkey");
                    switch (request.Query.type)
                    {
                        case "SearchEdipi":
                        case "SearchByIdentifier":
                            SetQueryString(config, localContext, searchRequest, request.Query.criteria);
                            searchRequest.MessageId = Guid.NewGuid().ToString();
                            searchRequest.OrganizationName = config.OrgName;
                            searchRequest.UserId = localContext.WorkflowExecutionContext.InitiatingUserId;
                            searchRequest.Debug = false;
                            searchRequest.LogSoap = config.LogSoap;
                            searchRequest.LogTiming = config.LogTimer;
                            
                            VeisRawRequest.Set(localContext.ActivityContext, JsonHelper.Serialize(searchRequest));
                            string message = request.Query.type == "SearchEdipi" ? "SearchMessage" : "SearchPerson";
                            string searchResponse = WebApiUtility.SendReceiveVeisRequest<string>(localContext, config, message, searchRequest);
                            VeisRawResponse.Set(localContext.ActivityContext, searchResponse);
                            Map(localContext, request.Query.criteria, JsonHelper.Deserialize<PersonSearchResponse>(searchResponse), config);
                            break;
                        case "SelectedPersonSearch":
                            var selectedRequest = new SelectedPersonRequest();
                            SetQueryString(config, localContext, selectedRequest, request.Query.criteria);
                            selectedRequest.OrganizationName = config.OrgName;
                            selectedRequest.UserId = localContext.WorkflowExecutionContext.InitiatingUserId;
                            selectedRequest.MessageId = Guid.NewGuid().ToString();
                            selectedRequest.LogSoap = config.LogSoap;
                            selectedRequest.LogTiming = config.LogTimer;
                            selectedRequest.RecordSource = "MVI";
                            selectedRequest.AssigningAuthority = "USVHA";
                            selectedRequest.CorrelationId = Guid.Empty;
                            Entity user = TryGetUserInfo(config, localContext);
                            if (user != null)
                            {
                                selectedRequest.UserFirstName = user["FirstName"].ToString();
                                selectedRequest.UserLastName = user["LastName"].ToString();
                                config.UserName = selectedRequest.UserFirstName + " " + selectedRequest.UserLastName;
                            }
                            VeisRawRequest.Set(localContext.ActivityContext, JsonHelper.Serialize(selectedRequest));
                            string selectedResponse = WebApiUtility.SendReceiveVeisRequest<string>(localContext, config, "SelectedPerson", selectedRequest);
                            VeisRawResponse.Set(localContext.ActivityContext, selectedResponse);
                            MapCorrespondingIds(config, localContext, JsonHelper.Deserialize<CorrespondingIdsResponse>(selectedResponse), selectedRequest);

                            break;

                        case "DeterministicSearch":
                            var deterministicRequest = new DeterministicSearchRequest();
                            SetQueryStringDeterministic(config, localContext, deterministicRequest, request.Query.criteria);
                            deterministicRequest.OrganizationName = config.OrgName;
                            deterministicRequest.UserId = localContext.WorkflowExecutionContext.InitiatingUserId;
                            deterministicRequest.MessageId = Guid.NewGuid().ToString();

                            var deterministicResponse = WebApiUtility.SendReceiveVeisRequest<PersonSearchResponse>(localContext, config, "DeterministicSearch", deterministicRequest);
                            VeisRawResponse.Set(localContext.ActivityContext, JsonHelper.Serialize(deterministicRequest));
                            Map(localContext, request.Query.criteria, (PersonSearchResponse)deterministicResponse, config);

                            //var peopleFound = ((deterministicResponse.Person != null) && (deterministicResponse.Person.Length > 0)) ? true : false;
                            //Logger.WriteDebugMessage(string.Format("DEBUG::" + mviSearchType + ": Search Response Message: People Found: {0}", peopleFound));

                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                throw new InvalidPluginExecutionException(WebApiUtility.BuildExceptionMessage(ex));
            }
        }

        private void SetQueryStringDeterministic(VeisConfig config, ActionContext localContext, DeterministicSearchRequest request, List<Criteria> criteria)
        {
            try
            {
                if (criteria != null && criteria.Count > 0)
                {
                    if (localContext != null && localContext.WorkflowExecutionContext != null)
                    {
                        request.OrganizationName = localContext.WorkflowExecutionContext.OrganizationName;
                        request.UserId = localContext.WorkflowExecutionContext.UserId;
                        foreach (Criteria c in criteria)
                        {
                            if (c != null && c.attribute != null && c.value != null && c.value != "null")
                            {
                                switch (c.attribute.ToLower())
                                {
                                    case "vetnumber":
                                        request.SearchType = DeterministicSearchType.SocialSecurity;
                                        request.SocialSecurityNumber = c.value;
                                        break;
                                    case "edipi":
                                        request.SearchType = DeterministicSearchType.Edipi;
                                        request.EdiPi = c.value;
                                        break;
                                }
                            }
                        }
                        Entity user = TryGetUserInfo(config, localContext);
                        if (user != null)
                        {
                            request.UserFirstName = user["FirstName"].ToString();
                            request.UserLastName = user["LastName"].ToString();
                            config.UserName = request.UserFirstName + " " + request.UserLastName;
                        }
                        // adding Auditing
                        if (config.EnableAuditing)
                        {
                            auditRequest(config, localContext, request);
                        }
                        if (request.SocialSecurityNumber != null && request.SocialSecurityNumber.Contains("XXX"))
                        {
                            request.SocialSecurityNumber = "";
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                //Logger.WriteToFile("Unable to set Query String due to: {0}".Replace("{0}",ex.Message));
                throw new InvalidPluginExecutionException("Unable to set Query String due to: {0}".Replace("{0}", ex.Message));
            }
        }

        private void SetQueryString(VeisConfig config, ActionContext localContext, PersonSearchRequest request, List<Criteria> criteria)
        {
            try
            {
                if (criteria != null && criteria.Count > 0)
                {
                    if (localContext != null && localContext.WorkflowExecutionContext != null)
                    {
                        request.OrganizationName = localContext.WorkflowExecutionContext.OrganizationName;
                        request.FetchMessageProcessType = "remote";
                        request.UserId = localContext.WorkflowExecutionContext.UserId;
                        string street = string.Empty;
                        string city = string.Empty;
                        string state = string.Empty;
                        string zip = string.Empty;
                        foreach (Criteria c in criteria)
                        {
                            if (c != null && c.attribute != null && c.value != null && c.value != "null")
                            {
                                switch (c.attribute.ToLower())
                                {
                                    case "isattended":
                                        request.IsAttended = Boolean.Parse(c.value);
                                        break;
                                    case "classcode":
                                        request.IdentifierClassCode = c.value;
                                        break;
                                    case "firstname":
                                        request.FirstName = c.value;
                                        break;
                                    case "lastname":
                                        request.FamilyName = c.value;
                                        break;
                                    case "dobstring":
                                        request.BirthDate = c.value;
                                        break;
                                    case "vetnumber":
                                        request.SocialSecurityNumber = c.value;
                                        break;
                                    case "middlename":
                                        request.MiddleName = c.value;
                                        break;
                                    case "gender":
                                        request.Gender = c.value;
                                        break;
                                    case "maidenname":
                                        request.MMN = c.value;
                                        break;
                                    case "phone":
                                        request.PhoneNumber = c.value;
                                        break;
                                    case "homestreet":
                                        street = c.value;
                                        break;
                                    case "homecity":
                                        city = c.value;
                                        break;
                                    case "homestate":
                                        state = c.value;
                                        break;
                                    case "homezip":
                                        zip = c.value;
                                        break;
                                    case "birthcity":
                                        request.POBC = c.value;
                                        break;
                                    case "birthstate":
                                        request.POBS = c.value;
                                        break;
                                    case "edipi":
                                        request.Edipi = c.value;
                                        break;
                                }
                            }
                        }
                        if (!string.IsNullOrEmpty(street) && !string.IsNullOrEmpty(city) && !string.IsNullOrEmpty(state) && !string.IsNullOrEmpty(zip))
                        {
                            request.Address = $"{street} {city}, {state} {zip}";
                        }
                        Entity user = TryGetUserInfo(config, localContext);
                        if (user != null)
                        {
                            request.UserFirstName = user["FirstName"].ToString();
                            request.UserLastName = user["LastName"].ToString();
                            config.UserName = request.UserFirstName + " " + request.UserLastName;
                        }
                        // adding Auditing
                        if (config.EnableAuditing)
                        {
                            auditRequest(config, localContext, request);
                        }
                        if (request.SocialSecurityNumber != null && request.SocialSecurityNumber.Contains("XXX"))
                        {
                            request.SocialSecurityNumber = "";
                        }
                        if (request.BirthDate != null && request.BirthDate.Contains("XX/XX"))
                        {
                            request.BirthDate = "";
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                //Logger.WriteToFile("Unable to set Query String due to: {0}".Replace("{0}",ex.Message));
                throw new InvalidPluginExecutionException("Unable to set Query String due to: {0}".Replace("{0}", ex.Message));
            }
        }

        private void SetQueryString(VeisConfig config, ActionContext localContext, SelectedPersonRequest request, List<Criteria> criteria)
        {
            try
            {
                if (criteria != null && criteria.Count > 0)
                {
                    if (localContext != null && localContext.WorkflowExecutionContext != null)
                    {
                        foreach (Criteria c in criteria)
                        {
                            if (c != null && c.attribute != null && c.value != null && c.value != "null")
                            {
                                switch (c.attribute.ToLower())
                                {
                                    case "patientsearchidentifier":
                                        request.PatientSearchIdentifier = c.value;
                                        break;
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                //Logger.WriteToFile("Unable to set Query String due to: {0}".Replace("{0}",ex.Message));
                throw new InvalidPluginExecutionException("Unable to set Query String due to: {0}".Replace("{0}", ex.Message));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        ///// <param name="userId"></param>
        /// <returns></returns>
        private Entity TryGetUserInfo(VeisConfig config, ActionContext localContext)
        {
            Entity user = new Entity("systemuser");
            ColumnSet cols = null;
            // will vary here for VBA vs. VHA applications
            if (config.OrgName.ToUpper().Contains("VRE"))
            {
                cols = new ColumnSet("firstname", "lastname", "patsr_css_text", "new_sensitivitylevel", "patsr_stationid");
            }
            else
            {
                cols = new ColumnSet("firstname", "lastname");
            }

            Entity result = localContext.OrganizationService.Retrieve("systemuser", localContext.WorkflowExecutionContext.UserId, cols);

            if (result != null)
            {
                user["FirstName"] = result.Attributes.Contains("firstname") && result.Attributes["firstname"] != null ? result.Attributes["firstname"].ToString() : string.Empty;
                user["LastName"] = result.Attributes.Contains("lastname") && result.Attributes["lastname"] != null ? result.Attributes["lastname"].ToString() : string.Empty;
            }

            if (config.OrgName.ToUpper().Contains("VRE"))
            {
                if (result.Attributes.Contains("patsr_stationid"))
                {
                    EntityReference station = (EntityReference)result.Attributes["patsr_stationid"];
                    if (station != null)
                    {
                        int start = station.Name.IndexOf("(") + 1;
                        int len = station.Name.IndexOf(")") - start;
                        config.StationNumber = station.Name.Substring(start, len);
                    }
                }
                config.CssName = result.Attributes.Contains("patsr_css_text") && result.Attributes["patsr_css_text"] != null ? result.Attributes["patsr_css_text"].ToString() : string.Empty;
                config.UserSensLevel = result.Attributes.Contains("new_sensitivitylevel") && result.Attributes["new_sensitivitylevel"] != null ? result.Attributes["new_sensitivitylevel"].ToString() : string.Empty;
            }

            return user;
        }


        #region Person Search Mapping
        private void Map(ActionContext localContext, List<Criteria> criteria, PersonSearchResponse response, VeisConfig config)
        {
            Response responseObj = new Response()
            {
                Results = new List<Person>()
            };
            List<Person> personCollection = new List<Person>();
            // exception check
            if (response == null || (response.ExceptionOccured && (string.IsNullOrEmpty(response.Message) || (!response.Message.ToLower().Contains("correlation does not exist") && !response.Message.ToLower().Contains("unknown key identifier")))))
            {
                responseObj.ExceptionOccured = true;
                responseObj.ExceptionMessage = "An unexpected error occured during the MVI search. Please try again or contact your system administrator if the problem persists.";
                Response.Set(localContext.ActivityContext, JsonHelper.Serialize(responseObj));
            }
            else if (response.Person == null || response.Person.Length == 0 || response.Person[0] == null)
            {
                responseObj.ExceptionMessage = "Your search in MVI did not find any records matching the search criteria.";
                if (!string.IsNullOrEmpty(response.Message))
                {
                    responseObj.ReturnMessage = response.Message;
                }
                if (!string.IsNullOrEmpty(response.CORPDbMessage))
                {
                    responseObj.ReturnMessage += response.CORPDbMessage;
                }
                //  internal search deprecated 4/3/2019 - gonna train up users to use the stock Contact search instead.
                //QueryInternal(localContext, criteria, responseObj);
                Response.Set(localContext.ActivityContext, JsonHelper.Serialize(responseObj));
            }
            else
            {
                // for each person returned create a Person in CRM
                if (response.Person.Length > 0)
                {
                    string vbaSensitiveLevel = "";
                    List<VeteranEmployeeFlag> vHAVeteranEmployeeFlags = new List<VeteranEmployeeFlag>();
                    localContext.Trace("GetSensitiveInfo : {0}", config.GetSensitiveInfo.ToString());
                    if (config.GetSensitiveInfo)
                    {
                        List<string> strs = new List<string>();
                        strs.Add(TryGetICN(response.Person[0]));
                        string url = String.Format("{0}{1}", config.VeisConfiguration.SvcConfigInfo.SvcBaseUrl, config.EmployeeEndpoint);
                        string key = config.VeisConfiguration.SvcConfigInfo.ApimSubscriptionKey;
                        localContext.Trace("vHAVeteranEmployeeFlags: {0} key: {1}", url, key);
                        NonVetResponse nvResponse = getVHAVeteranEmployeeFlags(url, key, strs);
                        foreach (NVDatum datum in nvResponse.Data)
                        {
                            localContext.Trace("datum: {0}", JsonHelper.Serialize(datum));
                            if (datum.NationalId == null)
                            {
                                vHAVeteranEmployeeFlags.Add(new VeteranEmployeeFlag() { NationalId = "", Value = "NO" });
                            }
                            else if (!(datum.Veteran == null || datum.Veteran == "" ? false : !(datum.Veteran.ToUpper() == "YES")))
                            {
                                vHAVeteranEmployeeFlags.Add(new VeteranEmployeeFlag() { NationalId = datum.NationalId, Value = "NO" });
                            }
                            else if (!(!(datum.Veteran.ToUpper() == "NO") || datum.NewPersonIndicator == null ? true : !(datum.NewPersonIndicator.ToUpper() == "YES")))
                            {
                                vHAVeteranEmployeeFlags.Add(new VeteranEmployeeFlag() { NationalId = datum.NationalId, Value = "YES" });
                            }
                            else if ((!(datum.Veteran.ToUpper() == "NO") || datum.PrimaryEligibilityCode == null ? false : datum.PrimaryEligibilityCode.ToUpper() == "EMPLOYEE"))
                            {
                                vHAVeteranEmployeeFlags.Add(new VeteranEmployeeFlag() { NationalId = datum.NationalId, Value = "YES" });
                            }

                        }
                    }
                    
                    VeisRawResponse.Set(localContext.ActivityContext, "{ \"vetEmpFlags\": " + JsonHelper.Serialize(vHAVeteranEmployeeFlags.ToArray()) + "}");

                    foreach (var person in response.Person)
                    {
                        localContext.Trace("Initial Person Map");
                        try
                        {
                            #region map the person with data
                            Person newPerson = new Person();
                            newPerson.Id = Guid.NewGuid();
                            newPerson.personId = newPerson.Id;
                            newPerson.VHAVeteranEmployeeFlags = JsonHelper.Serialize(vHAVeteranEmployeeFlags.ToArray());

                            //  get MVIIndentifier, ICN, patient identifier
                            localContext.Trace("TryGetIdentifiers");
                            TryGetMviQueryParams(person, newPerson);
                           
                            // need to handle VBA sensitivity for each result
                            if ((!config.OrgName.ToUpper().Contains("VRE") ? true : !config.GetSensitiveInfo))
                            {
                                TryGetCorpInfo(person, newPerson);
                                newPerson.VeteranSensitivityLevel = person.VeteranSensitivityLevel;
                                newPerson.EDIPI = person.EdiPi;
                            }
                            else
                            {
                                getMVICorpPidBIRLSFN(config, localContext, newPerson);
                                vbaSensitiveLevel = "";
                                bool flag2 = true;
                                if (!(newPerson.ParticipantID != "" ? true : !(newPerson.FileNumber != "")))
                                {
                                    vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.FileNumber);
                                    flag2 = false;
                                    VeisRawResponse.Set(localContext.ActivityContext, "{\"flag2\":" + flag2.ToString() + ",\"1vbaSensitiveLevel\":" + vbaSensitiveLevel);
                                }
                                else if (newPerson.ParticipantID != "")
                                {
                                    vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.ParticipantID);
                                    VeisRawResponse.Set(localContext.ActivityContext, "{\"flag2\":" + flag2.ToString() + ",\"2vbaSensitiveLevel\":" + vbaSensitiveLevel);
                                }
                                if (vbaSensitiveLevel == "")
                                {
                                    if (!(!flag2 ? true : !(newPerson.FileNumber != "")))
                                    {
                                        vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.FileNumber);
                                        VeisRawResponse.Set(localContext.ActivityContext, "{\"flag2\":" + flag2.ToString() + ",\"3vbaSensitiveLevel\":" + vbaSensitiveLevel);
                                    }
                                    else if (newPerson.ParticipantID != "")
                                    {
                                        vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.ParticipantID);
                                        VeisRawResponse.Set(localContext.ActivityContext, "{\"flag2\":" + flag2.ToString() + ",\"4vbaSensitiveLevel\":" + vbaSensitiveLevel);
                                    }
                                }
                                if (vbaSensitiveLevel == "")
                                {
                                    vbaSensitiveLevel = "0";
                                }
                                newPerson.VeteranSensitivityLevel = vbaSensitiveLevel;
                                if (config.UserSensLevel == "")
                                {
                                    config.UserSensLevel = "0";
                                }
                                int num = Convert.ToInt32(config.UserSensLevel);
                                int num1 = Convert.ToInt32(vbaSensitiveLevel);
                                object[] objArray = new object[] { "DEBUG::User Level: ", num, ": Caller Level: ", num1 };
                                if (num < num1)
                                {
                                    //  TODO: mask ??
                                    //goto Label1;
                                }
                            }

                            localContext.Trace("NameList");
                            if (person.NameList != null)
                            {
                                var legalName = person.NameList.FirstOrDefault(v => v.NameType.Equals("Legal", StringComparison.OrdinalIgnoreCase));
                                var alias = person.NameList.FirstOrDefault(v => v.NameType.Equals("Alias", StringComparison.OrdinalIgnoreCase));

                                if (legalName != null)
                                {
                                    newPerson.FirstName = legalName.GivenName;
                                    newPerson.LastName = legalName.FamilyName;
                                    newPerson.MiddleName = legalName.MiddleName;
                                }
                                else
                                {
                                    legalName = person.NameList.FirstOrDefault();

                                    if (legalName != null)
                                    {
                                        newPerson.FirstName = legalName.GivenName;
                                        newPerson.LastName = legalName.FamilyName;
                                        newPerson.MiddleName = legalName.MiddleName;
                                    }
                                }

                                newPerson.Alias = TryGetAlias(alias);
                            }

                            newPerson.FullName = "";
                            newPerson.FullName = FullNameBuilder(newPerson.FirstName, newPerson.MiddleName, newPerson.LastName);
                            if (person.Address != null && person.Address.StreetAddressLine != "" && person.Address.State != "" && person.Address.City != "" && person.Address.State != "" && person.Address.PostalCode != "")
                                newPerson.FullAddress = person.Address.StreetAddressLine + " " + person.Address.City + " " + person.Address.State + " " + person.Address.PostalCode;
                            else
                                newPerson.FullAddress = person.FullAddress;
                            if (person.Address != null && person.Address.StreetAddressLine != "")
                            {
                                newPerson.StreetAddressLine = person.Address.StreetAddressLine;
                            }
                            if (person.Address != null && person.Address.City != "")
                            {
                                newPerson.City = person.Address.City;
                            }
                            if (person.Address != null && person.Address.State != "")
                            {
                                newPerson.State = person.Address.State;
                            }
                            if (person.Address != null && person.Address.PostalCode != "")
                            {
                                newPerson.PostalCode = person.Address.PostalCode;
                            }
                            newPerson.BranchOfService = person.BranchOfService;
                            newPerson.PrimaryPhone = person.PhoneNumber;
                            newPerson.RecordSource = person.RecordSource;
                            newPerson.Gender = person.GenderCode;
                            newPerson.DeceasedDate = person.DeceasedDate;
                            newPerson.IdentityTheft = person.IdentifyTheft;
                            newPerson.url = person.Url;
                            //added from FtP, works with assisting in clicking on the record for the 2nd (selectedpersonsearch) MVI call
                            newPerson.url = " & select=*&$filter=patsr_PatientMviIdentifier eq '" + newPerson.PatientMviIdentifier + "' and patsr_SearchType eq 'SelectedPersonSearch'";
                            newPerson.DOBString = person.BirthDate;
                            newPerson.VetNumber = person.SocialSecurityNumber;
                            if (config.EnableAuditing)
                            {
                                auditPersonResponse(config, localContext, newPerson);
                            }

                            // will either set or mask the SSN and DOB values
                            string isEmp = string.Empty;
                            if ((!config.GetSensitiveInfo ? false : vHAVeteranEmployeeFlags.Count > 0))
                            {
                                isEmp = this.isEmployee(newPerson.ICN, vHAVeteranEmployeeFlags);
                            }
                            HandleSensitivityMasking(config, person, newPerson, isEmp);

                            List<Facility> facilities = new List<Facility>();
                            if (person.CorrespondingIdList != null)
                            {
                                foreach (var correspondingId in person.CorrespondingIdList)
                                {
                                    // for Corp/BRLS add the FileNumber
                                    if (correspondingId.AssigningFacility.Contains("CORP") && correspondingId.AssigningAuthority == "USVBA" && correspondingId.IdentifierType == "PI")
                                    {
                                        newPerson.FileNumber = correspondingId.PatientIdentifier;
                                    }
                                    // attempt to get the EDIPI, if present
                                    else if (string.IsNullOrEmpty(newPerson.EDIPI) && correspondingId.IdentifierType == "NI")
                                    {
                                        //  take first 10 of ICN?
                                        newPerson.EDIPI = (correspondingId.PatientIdentifier.Length > 10) ? correspondingId.PatientIdentifier.Substring(0, 10) : correspondingId.PatientIdentifier;
                                    }

                                    if (!string.IsNullOrEmpty(correspondingId.AssigningFacility))
                                    {
                                        var facility = this.RetrieveFacility(localContext, correspondingId.AssigningFacility);
                                        if (facility != null)
                                        {
                                            facilities.Add(new Facility() { Name = facility.GetAttributeValue<string>("patsr_name"), Id = facility.Id });
                                        }
                                    }
                                }
                            }

                            newPerson.Facilities = facilities.ToArray();
                            newPerson.ReturnMessage = response.Message + response.CORPDbMessage;
                            responseObj.Results.Add(newPerson);

                        }
                        catch (Exception e)
                        {
                            responseObj.ExceptionOccured = true;
                            responseObj.ExceptionMessage = "ERROR::Error while mapping person from NameList: " + e.Message;
                        }
                        #endregion
                        Response.Set(localContext.ActivityContext, JsonHelper.Serialize(responseObj));
                    }
                }
            }
        }

        private Person MapSelectedPerson(ActionContext localContext, CorrespondingIdsResponse response, VeisConfig config)
        {
            string vbaSensitiveLevel = "";

            #region map the person with data
            Person newPerson = new Person();
            newPerson.Id = Guid.NewGuid();
            newPerson.personId = newPerson.Id;

            // need to handle VBA sensitivity for each result
            if (!string.IsNullOrEmpty(config.OrgName) && config.OrgName.ToUpper().Contains("VRE") && config.GetSensitiveInfo)
            {
                getMVICorpPidBIRLSFN(config, localContext, newPerson);

                bool usedPid = true;
                if (newPerson.ParticipantID == "" && newPerson.FileNumber != "")
                {
                    //vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.FileNumber, false);      // sending false will search by FN and not Pid
                    usedPid = false;
                }
                else if (newPerson.ParticipantID != "")
                    //vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.ParticipantID, true);

                // check if we got a value with one, if not try the other
                if (vbaSensitiveLevel == "")
                {
                    if (usedPid)
                    {
                        //vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.FileNumber, false);
                    }
                    //else
                        //vbaSensitiveLevel = getSensitivityLevelVBA(config, newPerson.ParticipantID, true);
                }

                if (string.IsNullOrEmpty(vbaSensitiveLevel))
                    vbaSensitiveLevel = "0";

                //newPerson.VeteranSensitivityLevel = vbaSensitiveLevel;
                int userLvl = Convert.ToInt32(config.UserSensLevel);
                int callerLvl = Convert.ToInt32(vbaSensitiveLevel);

                //Logger.WriteDebugMessage("LOG::User Level: " + userLvl + ": Caller Level: " + callerLvl);

                if (userLvl < callerLvl)
                { // user does not have access     
                  // log attempted access
                    auditSensitivity(localContext, config, response.FullName, false);

                    newPerson.FirstName = "Sensitive";
                    newPerson.LastName = "Record";
                }
            }
            else
            {
                newPerson.EDIPI = response.Edipi;
            }

            newPerson.FullName = "";
            newPerson.FullName = FullNameBuilder(newPerson.FirstName,
                newPerson.MiddleName, newPerson.LastName);

            newPerson.url = response.Url;
            //added from FtP, works with assisting in clicking on the record for the 2nd (selectedpersonsearch) MVI call
            newPerson.url = " & select=*&$filter=patsr_PatientMviIdentifier eq '" + newPerson.PatientMviIdentifier + "' and patsr_SearchType eq 'SelectedPersonSearch'";

            // handle auditing of all calls
            if (config.EnableAuditing)
            {
                auditPersonResponse(config, localContext, newPerson);
            }

            List<Facility> facilities = new List<Facility>();
            if (response.CorrespondingIdList != null)
            {
                foreach (var correspondingId in response.CorrespondingIdList)
                {
                    // for Corp/BRLS add the FileNumber
                    if (correspondingId.AssigningFacility.Contains("CORP") && correspondingId.AssigningAuthority == "USVBA" && correspondingId.IdentifierType == "PI")
                    {
                        newPerson.FileNumber = correspondingId.PatientIdentifier;
                        newPerson.IdentifierType = correspondingId.IdentifierType;
                        newPerson.IdentifierAssigningFacility = correspondingId.AssigningFacility;
                        newPerson.IdentifierAssigningAuthority = correspondingId.AssigningAuthority;
                    }
                    // attempt to get the EDIPI, if present
                    else if (string.IsNullOrEmpty(newPerson.EDIPI) && correspondingId.IdentifierType == "NI")
                    {
                        newPerson.EDIPI = correspondingId.PatientIdentifier;
                        newPerson.IdentifierType = correspondingId.IdentifierType;
                        newPerson.IdentifierAssigningFacility = correspondingId.AssigningFacility;
                        newPerson.IdentifierAssigningAuthority = correspondingId.AssigningAuthority;
                    }

                    if (!string.IsNullOrEmpty(correspondingId.AssigningFacility))
                    {
                        var facility = this.RetrieveFacility(localContext, correspondingId.AssigningFacility);
                        if (facility != null)
                        {
                            facilities.Add(new Facility() { Name = facility.GetAttributeValue<string>("patsr_name"), Id = facility.Id });
                        }
                    }
                }
            }
            newPerson.Facilities = facilities.ToArray();
            return newPerson;
        }
        #endregion
        private Entity RetrieveFacility(ActionContext localContext, string assigningFacility)
        {
            QueryByAttribute query = new QueryByAttribute
            {
                //change these fields as required for your implementation
                ColumnSet = new ColumnSet("patsr_facilityid", "patsr_name"),
                EntityName = "patsr_facility"
            };
            query.AddAttributeValue("patsr_facilitycode", assigningFacility);

            EntityCollection results = localContext.OrganizationService.RetrieveMultiple(query);
            if (results.Entities.Count > 0)
            {
                return results.Entities[0];
            }
            return null;
        }

        private void MapCorrespondingIds(VeisConfig config, ActionContext localContext, CorrespondingIdsResponse response, SelectedPersonRequest request)
        {
            //  map to facility
            if (response != null && !response.ExceptionOccured)
            {
                List<CorrespondingID> facilities = new List<CorrespondingID>();
                foreach(CorrespondingID facility in response.CorrespondingIdList)
                {
                    //if (facility.AssigningFacility.Substring(0, 1) != "1" && facility.AssigningFacility.Substring(0, 1) != "2")
                    //{
                        Entity retrievedFacility = RetrieveFacility(localContext, facility.AssigningFacility);
                        facility.FacilityId = (retrievedFacility != null ? retrievedFacility.Id : Guid.Empty);
                        facility.FacilityName = (retrievedFacility != null ? retrievedFacility.GetAttributeValue<string>("patsr_name") : "");
                        facilities.Add(facility);
                    //}
                }
                response.CorrespondingIdList = facilities.ToArray();
            }
            Response.Set(localContext.ActivityContext, JsonHelper.Serialize(response));
        }

        

        private string getKVPSetting(ActionContext localContext, string name)
        {
            /*
            QueryByAttribute query = new QueryByAttribute
            {
                //change these fields as required for your implementation
                ColumnSet = new ColumnSet("patsr_stringvalue_text"),
                EntityName = "patsr_keyvaluepair"
            };
            query.AddAttributeValue("patsr_name_text", name);

            EntityCollection results = localContext.OrganizationService.RetrieveMultiple(query);
            if (results.Entities.Count > 0)
            {
                return (results.Entities[0].Contains("patsr_stringvalue_text")) ? results.Entities[0]["patsr_stringvalue_text"].ToString() : string.Empty;
            }
            */
            return string.Empty;
        }

        private static string TryGetICN(PatientPerson person)
        {
            string empty;
            try
            {
                if ((person.CorrespondingIdList == null ? false : person.CorrespondingIdList.Length>0))
                {
                    CorrespondingID correspondingID = person.CorrespondingIdList.FirstOrDefault<CorrespondingID>((CorrespondingID v) => (v.AssigningAuthority == null || !v.AssigningAuthority.Equals("USVHA", StringComparison.InvariantCultureIgnoreCase) || v.AssigningFacility == null || !(v.AssigningFacility == "200M") || v.IdentifierType == null ? false : v.IdentifierType.Equals("NI", StringComparison.InvariantCultureIgnoreCase))) ?? ((IEnumerable<CorrespondingID>)person.CorrespondingIdList).FirstOrDefault<CorrespondingID>((CorrespondingID v) => (v.AssigningAuthority == null || !v.AssigningAuthority.Equals("USVHA", StringComparison.InvariantCultureIgnoreCase) || v.IdentifierType == null ? false : v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase)));
                    empty = (correspondingID != null ? correspondingID.PatientIdentifier : string.Empty);
                }
                else
                {
                    empty = string.Empty;
                }
            }
            catch (ArgumentNullException argumentNullException1)
            {
                ArgumentNullException argumentNullException = argumentNullException1;
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}", argumentNullException.Message));
            }
            catch (FaultException<OrganizationServiceFault> faultException1)
            {
                FaultException<OrganizationServiceFault> faultException = faultException1;
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}", faultException.Message));
            }
            catch (NullReferenceException nullReferenceException1)
            {
                NullReferenceException nullReferenceException = nullReferenceException1;
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}", nullReferenceException.Message));
            }
            catch (Exception exception1)
            {
                Exception exception = exception1;
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}", exception.Message));
            }
            return empty;
        }

        private NonVetResponse getVHAVeteranEmployeeFlags(string url, string key, List<string> list)
        //private List<KeyValuePair<string, string>> getVHAVeteranEmployeeFlags(string url, string key, List<string> list)
        {
            NonVetRequest request = new NonVetRequest()
            {
                ClientName = "VCCM",
                Format = "JSON",
                NationalIds = list.ToArray()
            };
            string str = JsonHelper.Serialize(request);
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(url);
            httpWebRequest.Headers.Add("Ocp-Apim-Subscription-Key", key);

            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "POST";
            using (StreamWriter streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(str);
            }

            WebResponse webResponse = httpWebRequest.GetResponse();
            Stream webStream = webResponse.GetResponseStream();
            StreamReader responseReader = new StreamReader(webStream);
            string responseString = responseReader.ReadToEnd();
            responseReader.Close();

            NonVetResponse nvResponse = JsonHelper.Deserialize<NonVetResponse>(responseString);
            /*
             * List<KeyValuePair<string, string>> keyValuePairs = new List<KeyValuePair<string, string>>();
            try
            {
                foreach (NVDatum datum in nvResponse.Data)
                {
                    if (datum.NationalId == null)
                    {
                        // no nationalid
                        keyValuePairs.Add(new KeyValuePair<string, string>("", "NO"));
                    }
                    else if (!(datum.Veteran == null || datum.Veteran == "" ? false : !(datum.Veteran.ToUpper() == "YES")))
                    {
                        keyValuePairs.Add(new KeyValuePair<string, string>(datum.NationalId, "NO"));
                    }
                    else if (!(!(datum.Veteran.ToUpper() == "NO") || datum.NewPersonIndicator == null ? true : !(datum.NewPersonIndicator.ToUpper() == "YES")))
                    { 
                        keyValuePairs.Add(new KeyValuePair<string, string>(datum.NationalId, "YES"));
                    }
                    else if ((!(datum.Veteran.ToUpper() == "NO") || datum.PrimaryEligibilityCode == null ? false : datum.PrimaryEligibilityCode.ToUpper() == "EMPLOYEE"))
                    {
                        keyValuePairs.Add(new KeyValuePair<string, string>(datum.NationalId, "YES"));
                    }
                }
            }
            catch (Exception ex)
            {
                //Exception exception = ex;
                //base.Logger.WriteDebugMessage(string.Concat("ERROR::Error pulling NonVet info from HDR with the supplied ICNs.", exception.Message));
            }
            return keyValuePairs;*/
            
            return nvResponse;
        }

        private string isEmployee(string ICN, List<VeteranEmployeeFlag> empList)
        {
            string value;
            int num = 0;
            while (true)
            {
                if (num > empList.Count)
                {
                    //base.Logger.WriteDebugMessage("DEBUG::Employee ICN No match found for employee logic.");
                    value = "";
                    break;
                }
                else if (!(empList[num].NationalId == ICN))
                {
                    num++;
                }
                else
                {
                    //Logger logger = base.Logger;
                    //string[] key = new string[] { "DEBUG::Employee ICN match found : ", null, null, null, null, null };
                    //VeteranEmployeeFlag item = empList[num];
                    //key[1] = item.Key;
                    //key[2] = " and ";
                    //key[3] = ICN;
                    //key[4] = " values match, returning: ";
                    //item = empList[num];
                    //key[5] = item.Value;
                    //logger.WriteDebugMessage(string.Concat(key));
                    value = empList[num].Value;
                    break;
                }
            }
            return value;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="aliasName"></param>
        /// <returns></returns>
        private void HandleSensitivityMasking(VeisConfig config, PatientPerson person, Person newPerson, string employee)
        {
            bool flag;
            if ((!config.GetSensitiveInfo || !(newPerson.ICN != "") || !(config.SensitiveEndpoint != "") ? true : config.OrgName.ToUpper().Contains("VRE")))
            {
                newPerson.DOBString = person.BirthDate;
                newPerson.VetNumber = person.SocialSecurityNumber;
            }
            else
            {
                newPerson.VeteranSensitivityLevel = this.getSensitivityLevelVHA(config, newPerson, employee);
                if (!config.OrgName.ToUpper().Contains("VCL"))
                {
                    flag = true;
                }
                else
                {
                    flag = (newPerson.VeteranSensitivityLevel == "true:false" || newPerson.VeteranSensitivityLevel == "true:true" ? false : !(newPerson.VeteranSensitivityLevel == "false:true"));
                }
                if (!flag)
                {
                    newPerson.DOBString = "XX/XX/XXXX";
                    newPerson.VetNumber = "XXXXXXXXX";
                    newPerson.PrimaryPhone = "(XXX) XXX-XXXX";
                    if ((person.Address == null || !(person.Address.StreetAddressLine != "") || !(person.Address.State != "") || !(person.Address.City != "") || !(person.Address.State != "") ? false : person.Address.PostalCode != ""))
                    {
                        if (person.Address.StreetAddressLine != string.Empty)
                        {
                            string[] streetAddressLine = new string[] { person.Address.StreetAddressLine, " ", person.Address.City, " ", person.Address.State, " ", person.Address.PostalCode };
                            if (!(string.Concat(streetAddressLine) != "   "))
                            {
                                newPerson.FullAddress = "";
                            }
                            else
                            {
                                newPerson.FullAddress = "XXXXXXXXX  XXXX, XX  XXXXX";
                                newPerson.StreetAddressLine = "XXXXXXXXX";
                                newPerson.City = "XXXX";
                                newPerson.State = "XX";
                                newPerson.PostalCode = "XXXXX";
                            }
                        }
                    }
                }
                else if ((newPerson.VeteranSensitivityLevel == "true:true" ? true : newPerson.VeteranSensitivityLevel == "false:true"))
                {
                    newPerson.DOBString = "XX/XX/XXXX";
                    newPerson.VetNumber = "XXXXXXXXX";
                }
            }
        }

        private static string TryGetAlias(Name aliasName)
        {
            try
            {
                if (aliasName == null)
                {
                    return string.Empty;
                }

                const string nameFormat = "{0} {1}";
                var alias = string.Format(nameFormat, aliasName.GivenName, aliasName.FamilyName);

                return alias;
            }
            catch (Exception)
            {
                return string.Empty;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="person"></param>
        /// <returns></returns>
        private static void TryGetCorpInfo(PatientPerson person, Person newPerson)
        {
            try
            {
                if (person.CorrespondingIdList == null || !person.CorrespondingIdList.Any())
                {
                    newPerson.ParticipantID = string.Empty;
                }
                else
                {

                    var patientNo = person.CorrespondingIdList.FirstOrDefault((v =>
                        v.AssigningAuthority != null &&
                        v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                        v.AssigningFacility != null && v.AssigningFacility == "200CORP" &&
                        v.IdentifierType != null &&
                        v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase))) ??
                                    person.CorrespondingIdList.FirstOrDefault((v =>
                                        v.AssigningAuthority != null &&
                                        v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                                        v.IdentifierType != null &&
                                        v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase)));

                    newPerson.ParticipantID = patientNo != null ? patientNo.PatientIdentifier : string.Empty;
                    //  newPerson.FileNumber = patientNo != null ? patientNo.PatientIdentifier : string.Empty;                 
                }
            }
            catch (ArgumentNullException ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (NullReferenceException ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (Exception ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
        }

        private static void TryGetMviQueryParams(PatientPerson person, Person newPerson)
        {
            try
            {
                if (person.CorrespondingIdList == null || !person.CorrespondingIdList.Any())
                {
                    newPerson.PatientMviIdentifier = string.Empty;
                    newPerson.ICN = string.Empty;
                }
                else
                {

                    var patientNo = person.CorrespondingIdList.FirstOrDefault((v =>
                        v.AssigningAuthority != null &&
                        v.AssigningAuthority.Equals("USVHA", StringComparison.InvariantCultureIgnoreCase) &&
                        v.AssigningFacility != null && v.AssigningFacility == "200M" &&
                        v.IdentifierType != null &&
                        v.IdentifierType.Equals("NI", StringComparison.InvariantCultureIgnoreCase))) ??
                                    person.CorrespondingIdList.FirstOrDefault((v =>
                                        v.AssigningAuthority != null &&
                                        v.AssigningAuthority.Equals("USVHA", StringComparison.InvariantCultureIgnoreCase) &&
                                        v.IdentifierType != null &&
                                        v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase)));

                    newPerson.PatientMviIdentifier = patientNo != null ? patientNo.RawValueFromMvi : string.Empty;
                    newPerson.ICN = patientNo != null ? patientNo.PatientIdentifier : string.Empty;
                    newPerson.ParticipantID = patientNo != null ? patientNo.PatientIdentifier : string.Empty;
                }
            }
            catch (ArgumentNullException ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                //   ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                //   ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (NullReferenceException ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
            catch (Exception ex)
            {
                //Logger.WriteToFile("Unable to Get MVI Query Params due to: {0}".Replace("{0}", ex.Message));
                throw new InvalidPluginExecutionException("Unable to Get MVI Query Params due to: {0}".Replace("{0}",
                    ex.Message));
            }
        }

        private string FormatDate(string date)
        {
            if (date != "" && !string.IsNullOrWhiteSpace(date) && date.Length == 8)
            {
                return date.Substring(4, 2) + "/" + date.Substring(6, 2) + "/" + date.Substring(0, 4);
            }

            return date;
        }

        /// <summary>
        /// Gets the sensitivity level from ESR
        /// </summary>
        /// <param name="url"></param>
        /// <param name="icn"></param>
        /// <returns></returns>
        private string getSensitivityLevelVHA(VeisConfig config, Person person, string employee)
        {
            string url = String.Format("{0}{1}{2}", config.VeisConfiguration.SvcConfigInfo.SvcBaseUrl, config.SensitiveEndpoint, person.ICN);
            //return url;
            string str;
            bool flag;

            HttpWebRequest apiRequest = (HttpWebRequest)WebRequest.Create(url);
            apiRequest.Headers.Add("Ocp-Apim-Subscription-Key", config.VeisConfiguration.SvcConfigInfo.ApimSubscriptionKey);
            apiRequest.Method = "GET";

            WebResponse webResponse = apiRequest.GetResponse();
            Stream webStream = webResponse.GetResponseStream();
            StreamReader responseReader = new StreamReader(webStream);
            string str1 = responseReader.ReadToEnd();
            responseReader.Close();
            //return str1;
            person.SensitivityLevelVHA = str1;
            ESRSensitiveObject eSRSensitiveObject = JsonHelper.Deserialize<ESRSensitiveObject>(str1);
            person.PreferredFacility = (eSRSensitiveObject.Data.Demographics != null ? eSRSensitiveObject.Data.Demographics.PreferredFacility : "");
            if (!(employee == string.Empty))
            {
                str = (!(employee == "YES") ? string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":false") : "true:true");
            }
            else if (!config.OrgName.Contains("VCL"))
            {
                str = ((eSRSensitiveObject.Data.EnrollmentDeterminationInfo == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type == null ? true : !(eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type.ToUpper() == "EMPLOYEE")) ? string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":false") : string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":true"));
            }
            else
            {
                if (eSRSensitiveObject.Data.EnrollmentDeterminationInfo == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type == null)
                {
                    flag = true;
                }
                else
                {
                    flag = (eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type.ToUpper() == "EMPLOYEE" ? false : !(eSRSensitiveObject.Data.EnrollmentDeterminationInfo.Veteran.ToUpper() == "FALSE"));
                }
                str = (flag ? string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":false") : string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":true"));
            }

            return str;


        }
        private string getSensitivityLevelVBA(VeisConfig config, string icn)
        {
            //https://DNS.URL/api/ESR/EnrollmentEligibilitySensitivity/1.0/json/CRMSensitiveData/1012991357V923306
            /*{"ErrorOccurred":false,
             * "ErrorMessage":null,
             * "Status":null,
             * "DebugInfo":null,
             * "Data":{
             *      "SensitivityInfo":{"SensityFlag":"false"},
             *      "EnrollmentDeterminationInfo":{"PrimaryEligibility":{"Type":"NSC"},"Veteran":"true"},
             *      "Demographics":{"PreferredFacility":"565 - FAYETTEVILLE VA MEDICAL CENTER"}
             *      }
             *  }

            */
            string url = String.Format("{0}{1}{2}",config.VeisConfiguration.SvcConfigInfo.SvcBaseUrl,config.SensitiveEndpoint, icn);
            string lvl = "";

            HttpWebRequest apiRequest = (HttpWebRequest)WebRequest.Create(url);
            apiRequest.Headers.Add("Ocp-Apim-Subscription-Key", config.VeisConfiguration.SvcConfigInfo.ApimSubscriptionKey);
            apiRequest.Method = "GET";

            WebResponse webResponse = apiRequest.GetResponse();
            Stream webStream = webResponse.GetResponseStream();
            StreamReader responseReader = new StreamReader(webStream);
            string apiStringResponse = responseReader.ReadToEnd();
            responseReader.Close();

            return apiStringResponse;
            //using (var client = new WebClient())
            //{
                //var response = client.DownloadString(url);
                //lvl =  response;
                //var serializer = new JavaScriptSerializer();
                //SensitiveObject ro = serializer.Deserialize<SensitiveObject>(response);
                //return lvl;
                //return (ro.Data[0].Return1 == null) ? "" : ro.Data[0].Return1.ScrtyLevelTypeCd;
            //}
            //Logger.WriteToFile("Sensitivity Level Not Found for PatientId: {0}".Replace("{0}", pId));
            //return "10";
        }

        /// <summary>
        /// This basically makes the 2nd MVI call to get the correlations to 
        /// get the FileNumber so we can get the sensitivity level and compare
        /// </summary>
        /// <param name="ICN"></param>
        /// <returns></returns>
        private void getMVICorpPidBIRLSFN(VeisConfig config, ActionContext localContext, Person newPerson)
        {
            SelectedPersonRequest selectedRequest = new SelectedPersonRequest();
            selectedRequest.OrganizationName = config.OrgName;
            selectedRequest.UserId = localContext.WorkflowExecutionContext.InitiatingUserId;
            selectedRequest.MessageId = Guid.NewGuid().ToString();
            //selectedRequest.Debug = false;
            selectedRequest.LogSoap = config.LogSoap;
            selectedRequest.LogTiming = config.LogTimer;
            selectedRequest.RecordSource = "MVI";
            selectedRequest.AssigningAuthority = "USVHA";
            selectedRequest.CorrelationId = Guid.Empty;
            Entity user = TryGetUserInfo(config, localContext);
            if (user != null)
            {
                selectedRequest.UserFirstName = user["FirstName"].ToString();
                selectedRequest.UserLastName = user["LastName"].ToString();
                config.UserName = selectedRequest.UserFirstName + " " + selectedRequest.UserLastName;
            }
            selectedRequest.PatientSearchIdentifier = newPerson.ICN;

            string selectedResponseString = WebApiUtility.SendReceiveVeisRequest<string>(localContext, config, "SelectedPerson", selectedRequest);
            CorrespondingIdsResponse selectedResponse = JsonHelper.Deserialize<CorrespondingIdsResponse>(selectedResponseString);


            if (selectedResponse.ExceptionOccured)
            {
                //Logger.WriteDebugMessage(string.Format("ERROR::VBASensitivityCheck: Search Exception Message: {0}", selectedResponse.RawMviExceptionMessage));
            }
            //else
            //    Logger.WriteDebugMessage(string.Format("LOG::VBASensitivityCheck: Success: {0}", selectedResponse.RawMviExceptionMessage));

            try
            {
                if (selectedResponse.CorrespondingIdList == null || !selectedResponse.CorrespondingIdList.Any())
                {
                    return;
                }
                else
                {

                    var patientNo = selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                        v.AssigningAuthority != null &&
                        v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                        v.AssigningFacility != null && v.AssigningFacility == "200CORP" &&
                        v.IdentifierType != null &&
                        v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase))) ??
                                    selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                                        v.AssigningAuthority != null &&
                                        v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                                        v.IdentifierType != null &&
                                        v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase)));
                    newPerson.ParticipantID = patientNo != null ? patientNo.PatientIdentifier : string.Empty;

                    var patientFN = selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                      v.AssigningAuthority != null &&
                      v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                      v.AssigningFacility != null && v.AssigningFacility == "200BRLS" &&
                      v.IdentifierType != null &&
                      v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase))) ??
                                  selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                                      v.AssigningAuthority != null &&
                                      v.AssigningAuthority.Equals("USVBA", StringComparison.InvariantCultureIgnoreCase) &&
                                      v.IdentifierType != null &&
                                      v.IdentifierType.Equals("PI", StringComparison.InvariantCultureIgnoreCase)));
                    newPerson.FileNumber = patientFN != null ? patientFN.PatientIdentifier : string.Empty;

                    //999872003^NI^200DOD^USDOD}
                    var patientEDIPI = selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                     v.AssigningAuthority != null &&
                     v.AssigningAuthority.Equals("USDOD", StringComparison.InvariantCultureIgnoreCase) &&
                     v.AssigningFacility != null && v.AssigningFacility == "200DOD" &&
                     v.IdentifierType != null &&
                     v.IdentifierType.Equals("NI", StringComparison.InvariantCultureIgnoreCase))) ??
                                 selectedResponse.CorrespondingIdList.FirstOrDefault((v =>
                                     v.AssigningAuthority != null &&
                                     v.AssigningAuthority.Equals("USDOD", StringComparison.InvariantCultureIgnoreCase) &&
                                     v.IdentifierType != null &&
                                     v.IdentifierType.Equals("NI", StringComparison.InvariantCultureIgnoreCase)));

                    newPerson.EDIPI = patientEDIPI != null ? patientEDIPI.PatientIdentifier : string.Empty;

                    //Logger.WriteDebugMessage(string.Format("LOG::VBASensitivityCheck: Response Success for PId: {0}, FileNumber: {1}, EDIPI: {2} ", patientNo.PatientIdentifier, patientFN.PatientIdentifier, patientEDIPI.PatientIdentifier));
                }
            }
            catch (Exception ex)
            {
                //Logger.WriteDebugMessage("ERROR::VBASensitivityCheck: Error in getMVICorpPidBIRLSFN()");
            }
        }
        /// <summary>
        /// Will return a string of the response from MVI
        /// </summary>
        /// <param name="count"></param>
        /// <param name="person"></param>
        /// <returns></returns>
        private void auditPersonResponse(VeisConfig config, ActionContext localContext, Person person)
        {
            if (string.IsNullOrEmpty(config.AuditUrl))
            {
                config.AuditUrl = getKVPSetting(localContext, "MVI_Log");
            }

            string tempICN = person.ICN;
            if (tempICN.Length > 15)
                tempICN = person.ICN.Substring(0, 15);

            var MviLog = new MVILog
            {
                Address = person.FullAddress,
                Attended = person.IsAttended,
                ClassCode = person.ClassCode,
                CreatedOn = DateTime.UtcNow,
                DOB = person.DOBString,
                EDIPI = person.EDIPI,
                First = person.FirstName,
                ICN = tempICN,
                Last = person.LastName,
                Middle = person.MiddleName,
                MMN = person.MMN,
                Organization_Name = config.OrgName,
                Phone = person.PrimaryPhone,
                POBC = person.POBC,
                POBS = person.POBS,
                RequestType = person.SearchType,
                SessionId = new Guid().ToString(),
                Sex = person.Gender,
                Source = person.RecordSource,
                Type = "Response",
                UserName = config.UserName,
                SSN = person.VetNumber
            };

            var json = new JavaScriptSerializer().Serialize(MviLog);
            handleRequest(config.AuditUrl, json, "MVILog");
        }
        private void auditSensitivity(ActionContext localContext, VeisConfig config, string crmeFullName, bool accessed)
        {
            string sensitiveUrl = getKVPSetting(localContext, "MVI_Sensitivity_Log");

            var Log = new SensitiveLog()
            {
                Accessed = accessed,
                Access_Time = DateTime.UtcNow.ToString("G"),
                HDR_Response_Code = "",
                Organization_Name = config.OrgName.Trim(),
                Sensitive_Veteran = true,
                SessionId = new Guid().ToString(),
                Veteran = crmeFullName,
                Who_Accessed = config.UserName
            };

            var json = new JavaScriptSerializer().Serialize(Log);
            handleRequest(sensitiveUrl, json, "Sensitivity_Log");
        }

        /// <summary>
        /// Will handle adding the request details into a string to be kept in a log record
        /// </summary>
        /// <param name="request">PersonSearchRequest object</param>
        /// <returns>string of the requet</returns>
        private void auditRequest(VeisConfig config, ActionContext localContext, PersonSearchRequest request)
        {
            if (string.IsNullOrEmpty(config.AuditUrl))
            {
                config.AuditUrl = getKVPSetting(localContext, "MVI_Log");
            }
            bool attended = request.IsAttended;

            var Log = new MVILog
            {
                Address = request.Address,
                Attended = attended,
                ClassCode = request.IdentifierClassCode,
                CreatedOn = DateTime.UtcNow,
                DOB = request.BirthDate,
                EDIPI = request.Edipi,
                First = request.FirstName,
                ICN = request.PatientIdentifier,
                Last = request.FamilyName,
                Middle = request.MiddleName,
                MMN = request.MMN,
                Organization_Name = config.OrgName,
                Phone = request.PhoneNumber,
                POBC = request.POBC,
                POBS = request.POBS,
                RequestType = "PersonSearchRequest",
                SessionId = new Guid().ToString(),
                Sex = request.Gender,
                Source = "",
                SSN = request.SocialSecurityNumber,
                Type = "Request",
                UserName = config.UserName
            };

            var json = new JavaScriptSerializer().Serialize(Log);
            //TODO handleRequest(config.auditUrl, json, "MVILog");
        }
        private void auditRequest(VeisConfig config, ActionContext localContext, SelectedPersonRequest request)
        {
            if (config.AuditUrl == "")
            {
                config.AuditUrl = getKVPSetting(localContext, "MVI_Log");
            }
            //bool attended = request.IsAttended;

            var Log = new MVILog
            {
                //Address = request.Address,
                //Attended = attended,
                //ClassCode = request.IdentifierClassCode,
                CreatedOn = DateTime.UtcNow,
                //DOB = request.BirthDate,
                //EDIPI = request.Edipi,
                //First = request.FirstName,
                //ICN = request.PatientIdentifier,
                //Last = request.FamilyName,
                //Middle = request.MiddleName,
                //MMN = request.MMN,
                Organization_Name = config.OrgName,
                //Phone = request.PhoneNumber,
                //POBC = request.POBC,
                //POBS = request.POBS,
                RequestType = "SelectedPersonRequest",
                SessionId = new Guid().ToString(),
                //Sex = request.Gender,
                Source = "",
                //SSN = request.SocialSecurityNumber,
                Type = "Request",
                UserName = config.UserName
            };

            var json = new JavaScriptSerializer().Serialize(Log);
            //TODO handleRequest(config.auditUrl, json, "MVILog");
        }
        private void auditRequest(VeisConfig config, ActionContext localContext, DeterministicSearchRequest request)
        {
            if (string.IsNullOrEmpty(config.AuditUrl))
            {
                config.AuditUrl = getKVPSetting(localContext, "MVI_Log");
            }

            var Log = new MVILog
            {
                CreatedOn = DateTime.UtcNow,
                DOB = request.BirthDate,
                EDIPI = request.EdiPi,
                Organization_Name = config.OrgName,
                RequestType = "DeterministicSearchRequest",
                SessionId = new Guid().ToString(),
                Source = "",
                SSN = request.SocialSecurityNumber,
                Type = "Request",
                UserName = config.UserName
            };

            var json = new JavaScriptSerializer().Serialize(Log);
            //TODO handleRequest(config.auditUrl, json, "MVILog");
        }

        /// <summary>
        /// Handles sending the requests and responses for audit logging and sensitivity
        /// </summary>
        /// <param name="url">The url of the service</param>
        /// <param name="json">the json string of the object</param>
        private void handleRequest(string url, string json, string logTable)
        {
            /* TODO
            var httpWebRequest = (HttpWebRequest)WebRequest.Create(url);
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Method = "POST";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(json);
                streamWriter.Flush();
                streamWriter.Close();
            }

            try
            {
                var httpResponse = (HttpWebResponse)httpWebRequest.GetResponse();
                using (var streamReader = new StreamReader(httpResponse.GetResponseStream()))
                {
                    streamReader.ReadToEnd();
                    streamReader.Close();
                }
                httpResponse.Close();
            }
            catch (Exception e)
            {
                //Logger.WriteDebugMessage("ERROR::Error creating " + logTable + " response record: " + e.Message);
            }
            */
        }

        private string FullNameBuilder(string first, string middle, string last)
        {
            string fullName = "n/a";

            var fName = (string.IsNullOrEmpty(first)) ? "" : first;
            var mName = (string.IsNullOrEmpty(middle)) ? "" : middle;
            var lName = (string.IsNullOrEmpty(last)) ? "" : last;

            fullName = string.Format("{0} {1} {2}", fName, mName, lName);

            return fullName;
        }
    }

    public class Response
    {
        public bool ExceptionOccured = false;
        public string ExceptionMessage;
        public string ReturnMessage;
        public List<Person> Results;
    }

    public class VeisConfig
    {
        public string AuditUrl { get; set; }
        public string OrgOverride { get; set; }
        public string OrgName { get; set; }
        public bool GetSensitiveInfo { get; set; }
        public string EmployeeEndpoint { get; set; }
        public string SensitiveEndpoint { get; set; }
        public bool EnableAuditing { get; set; }
        public bool LogTimer { get; set; }
        public bool LogSoap { get; set; }
        public string UserSensLevel { get; set; }
        public string UserName { get; set; }
        public string StationNumber { get; set; }
        public string CssName { get; set; }
        public VeisConfiguration VeisConfiguration { get; set; }
    }


    internal class SensitiveLog
    {
        public string Organization_Name;
        public string Veteran;
        public bool Sensitive_Veteran;
        public bool Accessed;
        public string Who_Accessed;
        public string HDR_Response_Code;
        public string Access_Time;
        public string SessionId;
        public string userName;
    }

    internal class MVILog
    {
        public string Organization_Name;
        public string First;
        public string Last;
        public string Middle;
        public string DOB;
        public string SSN;
        public string EDIPI;
        public string Phone;
        public bool Attended;
        public string Address;
        public string Sex;
        public string MMN;
        public string POBC;
        public string POBS;
        public string RequestType;
        public string ICN; //15
        public string Source;
        public string ClassCode;
        public string Type;
        public DateTime CreatedOn;
        public string SessionId;
        public string UserName;
    }
    internal class MVILog2
    {
        public string Organization_Name;
        public string DOB;
        public string SSN;
        public string EDIPI;
        public string Phone;
        public string ICN;
        public string IDs;
        public DateTime CreatedOn;
        public string SessionId;
        public string UserName;
    }
    public class SensitivityInfo
    {
        public string SensityFlag { get; set; }
    }

    public class PrimaryEligibility
    {
        public string Type { get; set; }
    }

    public class EnrollmentDeterminationInfo
    {
        public PrimaryEligibility PrimaryEligibility { get; set; }
        public string Veteran { get; set; }
    }

    public class Demographics
    {
        public string PreferredFacility { get; set; }
    }

    public class ESRSensitiveObject
    {
        public bool ErrorOccurred { get; set; }
        public string ErrorMessage { get; set; }
        public string Status { get; set; }
        public string DebugInfo { get; set; }
        public ESRData Data { get; set; }
    }
    public class ESRData
    {
        public SensitivityInfo SensitivityInfo { get; set; }
        public EnrollmentDeterminationInfo EnrollmentDeterminationInfo { get; set; }
        public Demographics Demographics { get; set; }
    }

                /*{"ErrorOccurred":false,
             * "ErrorMessage":null,
             * "Status":null,
             * "DebugInfo":null,
             * "Data":{
             *      "SensitivityInfo":{"SensityFlag":"false"},
             *      "EnrollmentDeterminationInfo":{"PrimaryEligibility":{"Type":"NSC"},"Veteran":"true"},
             *      "Demographics":{"PreferredFacility":"565 - FAYETTEVILLE VA MEDICAL CENTER"}
             *      }
             *  }
             */
    

    public class SensitiveObject
    {
        public bool ErrorOccurred { get; set; }
        public object ErrorMessage { get; set; }
        public object Status { get; set; }
        public object DebugInfo { get; set; }
        public List<Datum> Data { get; set; }
    }
    public class Datum
    {
        public object Fault { get; set; }
        public SensitiveInfo Return1 { get; set; }
    }
    public class SensitiveInfo
    {
        public string BdnStnNbr { get; set; }
        public string BirlsFolderLctnTxt { get; set; }
        public string Cd { get; set; }
        public string DgnstcSuprsnInd { get; set; }
        public string FcltyTypeCd { get; set; }
        public DateTime JrnDt { get; set; }
        public string JrnLctnId { get; set; }
        public string JrnObjId { get; set; }
        public string JrnStatusTypeCd { get; set; }
        public string JrnUserId { get; set; }
        public string PtcpntId { get; set; }
        public string RmksTxt { get; set; }
        public string ScrtyLevelTypeCd { get; set; }
        public string SntvtyLevelId { get; set; }
    }
    public class VeteranEmployeeFlag
    {
        public string NationalId { get; set; }
        public string Value { get; set; }
    }

}
#endregion