﻿using System;
using VEIS.Core.Core;
using VEIS.Core.Messages;
using VEIS.Core.Processor;

namespace VEIS.Core.Handlerbase
{ 
    public static class EcHandlerBase
    {
        /// <summary>
        /// This should be called by all Handlers as the entire HandleRequestResponse body - see remarks for example
        /// </summary>
        /// <typeparam name="T1">Type of ECProcessor</typeparam>
        /// <typeparam name="T2">Type of VEISEcResponse</typeparam>
        /// <typeparam name="T3">Type of VEISEcRequest</typeparam>
        /// <param name="message">Request message of type T3</param>
        /// <param name="caller">This should be the Handler Class that receives the VINEXT Message and "handles" it by calling this method</param>
        /// <returns>returns the response message as specified in the Process of the ProcessorImplementation.Process or the error messaging surrounding it</returns>
        /// <remarks>e.g. return EcHandlerBase.HandleMessage[VEISsMSsearchMediSpanRequest, searchMediSpanProcessor, VEISsMSsearchMediSpanResponse]((VEISsMSsearchMediSpanRequest)message, this);</remarks>
        public static IMessageBase HandleMessage<T1, T2, T3>(T1 message, RequestResponseHandler caller)
            where T1 : VEISEcRequestBase
            where T2 : EcProcessorBase, new()
            where T3 : VEISEcResponseBase, new()
        {
            VEISEcResponseBase response = new T3();
            try
            {
                caller.LogMessageReceipt(message);
                T2 processor = new T2();
                response = processor.Execute<T3>(message);
                Logger.Instance.Info($"Successfully Completed Processor {processor.GetType().Name} for {response.GetType().Name} response.");
                return response;
            }
            catch (Exception ex)
            {
                response = new T3
                {
                    ExceptionOccurred = true,
                    ExceptionMessage = ex.Message,
                    EcTraceLog = ex.StackTrace,
                };
                try
                {
                    Logger.Instance.Error($"{ex.Message}.  Full Stack Trace: {ex.StackTrace}");
                }
                catch (Exception ex2)
                {
                    response.EcTraceLog = $"Failed to write Logger to destination: {ex2.Message}.  Error Trace: {response.EcTraceLog}.";
                }
                return response;
            }
            finally
            {
                caller.LogMessageResponse(response);
            }
        }
    }
}
