﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VEIS.Core.Core;
using VEIS.Core.Messages;
using VEIS.Core.Wcf;

namespace VEIS.Core.Processor
{

    public abstract class EcProcessorBase
    {
        public StringBuilder TraceString { get; set; }
        //public VEISEcRequestBase RequestBase { get; set; }
        public VEISEcResponseBase ResponseBase { get; set; }
        public Stopwatch Stopwatch { get; set; }
        public virtual VEISEcResponseBase Execute<T>(VEISEcRequestBase request)
            where T : VEISEcResponseBase, new()
        {
            Stopwatch = Stopwatch.StartNew();
            SoapLog.Current.Active = true;
            VEISEcResponseBase response = new T();
            TraceString = new StringBuilder();
            Trace($"{Environment.MachineName} box, {GetType().Name} processor received {request?.GetType().Name} message from org: {request?.OrganizationName}. Id: {request?.MessageId}", LogLevel.Info);
            try
            {
                if (request == null)
                {
                    var error = $"EC {GetType().FullName} recieved a null message";
                    Trace(error, LogLevel.Error);
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = error;
                    return response;
                }
                else
                {
                    Trace($"Begin Processing MessageId:{request.MessageId}");
                    response = Process(request);
                    if (response.ExceptionOccurred || !String.IsNullOrEmpty(response.ExceptionMessage))
                        Trace($"EC {GetType().Name} returned failure message: {response.ExceptionMessage}", LogLevel.Warn);
                    else
                        Trace($"EC {GetType().Name} returned success");
                    Trace($"Completed Processing {GetType().FullName} for Message Type {response.GetType().Name} with Id: {response.MessageId}", LogLevel.Info);
                    return response;
                }
            }
            catch (Exception ex)
            {
                Trace($"{GetType().FullName} threw an uncaught exception: {ex.Message}: Stack Trace: {ex.StackTrace}", LogLevel.Error);
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                return response;
            }
            finally
            {
                response.EcTraceLog = TraceString.ToString();
                response.ProcessorTimer = Stopwatch.ElapsedMilliseconds;
                response.MessageId = request.MessageId;
                response.CorrelationId = request.CorrelationId;
                Stopwatch.Stop();
                ProcessorHelpers.TrackEcStatistics(request, response, GetType().Name);
            }
        }

        /// <summary>
        /// write to log.net and add to the trace string
        /// </summary>
        /// <param name="message">message to output</param>
        /// <param name="level">1 = error, 2 = warning, 3 = info, default = debug</param>
        public virtual void Trace(string message, int level = 4)
        {
            Trace(message, (LogLevel)level);
        }

        /// <summary>
        /// write to log.net and add to the trace string
        /// </summary>
        /// <param name="message">message to output</param>
        /// <param name="level"></param>
        public virtual void Trace(string message, LogLevel level)
        {
            TraceString.AppendLine($"{level.ToString()}:{Stopwatch.ElapsedMilliseconds}ms:{message}");
            switch (level)
            {
                case LogLevel.Error:
                    Logger.Instance.Error(message);
                    break;
                case LogLevel.Warn:
                    Logger.Instance.Warn(message);
                    break;
                case LogLevel.Info:
                    Logger.Instance.Info(message);
                    break;
                case LogLevel.Debug:
                case LogLevel.Unknown:
                default:
                    Logger.Instance.Debug(message);
                    break;
            }
        }

        public string GetRequestSoap(string soapLog)
        {
            throw new NotImplementedException();
            //return soapLog;
        }

        /// <summary>
        /// Class must be overridden with the details of the specific processor
        /// </summary>
        /// <param name="request">the request coming in from the plugin</param>
        /// <returns>The primary response to go back to the plugin</returns>
        public abstract VEISEcResponseBase Process(VEISEcRequestBase request);
    }

    public enum LogLevel
    {
        Unknown,
        Error,
        Warn,
        Info,
        Debug
    }
}
