﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VEIS.Core.Messages;

namespace VEIS.Core.Processor
{

    public static class ProcessorHelpers
    {
        public static string Serialize(object obj)
        {
            using (var memoryStream = new MemoryStream())
            {
                var serializer = new System.Xml.Serialization.XmlSerializer(obj.GetType());
                serializer.Serialize(memoryStream, obj);

                return Encoding.UTF8.GetString(memoryStream.ToArray());
            }
        }

        public static string SanitizeString(string input, ICollection<string> badWords)
        {
            foreach (var badWord in badWords)
            {
                while (input.Contains(badWord))
                    input = EliminateSensitiveInfo(input, badWord);
            }
            return input;
        }
        private static string EliminateSensitiveInfo(string input, string match)
        {
            if (input.Contains(match))
            {
                var isJSON = input[input.IndexOf(match) - 1] == '\"';
                var startIndex = input.IndexOf(match) + match.Length + 1;
                int endIndex = 0;
                if (isJSON)
                    endIndex = input.IndexOf("\",", startIndex) + 1;
                else
                    endIndex = input.IndexOf(match, startIndex) - 2;
                input = input.Remove(startIndex, endIndex - startIndex);
                return input.Replace(match, "REPLACED_PII");
            }
            else
                return input;
        }
        /// <summary>
        /// Write the logs to whatever Common Tracking location is decided on
        /// </summary>
        /// <param name="request"></param>
        /// <param name="response"></param>
        public static void TrackEcStatistics(VEISEcRequestBase request, VEISEcResponseBase response, string processorName)
        {
            var tracker = new MessageTracker(request, response, processorName);
            tracker.Track();
        }
    }

    public class MessageTracker
    {
        public MessageTracker(VEISEcRequestBase ecRequest, VEISEcResponseBase ecResponse, string name)
        {
            ProcessorName = name;
            MessageType = "EC";
            RequestType = ecRequest?.GetType().Name;
            ResponseType = ecResponse?.GetType().Name;
            if (!string.IsNullOrEmpty(RequestType))
            {
                RequestType = TrimLength(RequestType, 100);
                CorrelationId = ecRequest.CorrelationId;
                MessageId = ecRequest.MessageId;
                CallingOrganization = ecRequest.OrganizationName;
            }
            if (!string.IsNullOrEmpty(ResponseType))
            {
                ResponseType = TrimLength(ResponseType, 100);
                ProcessorTimer = ecResponse.ProcessorTimer;
                EcTimer = ecResponse.ServiceTimer;
                ExceptionOccurred = ecResponse.ExceptionOccurred;
                ExceptionMessage = string.IsNullOrEmpty(ecResponse.ExceptionMessage) ? string.Empty :
                    TrimLength(ecResponse.ExceptionMessage, 100);
            }
        }
        public MessageTracker(RequestBase request, ResponseBase response, string name)
        {
            ProcessorName = name;
            MessageType = "LOB";
            RequestType = request?.GetType().Name;
            ResponseType = response?.GetType().Name;
            if (!string.IsNullOrEmpty(RequestType))
            {
                RequestType = TrimLength(RequestType, 100);
                MessageId = request.MessageId;
                CallingOrganization = request.OrganizationName;
            }
            if (!string.IsNullOrEmpty(ResponseType))
            {
                ResponseType = TrimLength(ResponseType, 100);
                EcTimer = Convert.ToInt64(response.EcDiagnostics?.Timer);
                ProcessorTimer = Convert.ToInt64(response.ProcessorTimer);
                ExceptionOccurred = response.ExceptionOccured;
                ExceptionMessage = string.IsNullOrEmpty(response.ExceptionMessage) ? string.Empty :
                    TrimLength(response.ExceptionMessage, 100);
            }
        }
        /// <summary>
        /// This is a placeholder method for what will write the data to Azure Tracking Mechanism
        /// </summary>
        internal void Track() { WriteToDataStore(); }

        private void WriteToDataStore()
        {
            /* Write to some db asynchronously-Not Yet Implemented */
        }

        /// <summary>
        /// Ensures strings dont exceed the max number of characters, truncate them at max length if they do
        /// </summary>
        /// <param name="input">string to be trimmed</param>
        /// <param name="length">length at which to trim</param>
        /// <returns>trimmed string</returns>
        private string TrimLength(string input, int length)
        {
            if (string.IsNullOrEmpty(input) || input.Length <= length)
                return input;
            else
                return input.Substring(0, length);
        }
        string RequestType { get; set; }
        string ResponseType { get; set; }
        string ProcessorName { get; set; }
        long EcTimer { get; set; }
        long ProcessorTimer { get; set; }
        bool ExceptionOccurred { get; set; }
        string ExceptionMessage { get; set; }
        string CallingOrganization { get; set; }
        Guid CorrelationId { get; set; }
        string MessageId { get; set; }
        string MessageType { get; set; }
    }
}