﻿using System;
using System.IO;
using System.Linq;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel;
using System.Configuration;
using System.ServiceModel.Configuration;
using System.Security.Cryptography.X509Certificates;
using System.Reflection;
using System.Collections.Concurrent;

namespace VEIS.Core.Wcf
{
    public class CustomClientChannel<T> : ChannelFactory<T>
    {
        readonly string _ConfigurationPath;
        readonly string _EndpointConfigurationName;
        private static ConcurrentDictionary<string, ServiceModelSectionGroup> _sectionDictionary = new ConcurrentDictionary<string, ServiceModelSectionGroup>();

        public CustomClientChannel(string configurationPath)
            : base(typeof(T))
        {
            _ConfigurationPath = configurationPath;

            InitializeEndpoint((string)null, null);
        }

        public CustomClientChannel(Binding binding, string configurationPath)
            : this(binding, (EndpointAddress)null, configurationPath)
        {
        }

        public CustomClientChannel(ServiceEndpoint serviceEndpoint, string configurationPath)
            : base(typeof(T))
        {
            _ConfigurationPath = configurationPath;

            InitializeEndpoint(serviceEndpoint);
        }

        public CustomClientChannel(string endpointConfigurationName, string configurationPath)
            : this(endpointConfigurationName, null, configurationPath)
        {
        }

        public CustomClientChannel(Binding binding, EndpointAddress endpointAddress, string configurationPath)
            : base(typeof(T))
        {
            _ConfigurationPath = configurationPath;

            InitializeEndpoint(binding, endpointAddress);
        }

        public CustomClientChannel(Binding binding, string remoteAddress, string configurationPath)
            : this(binding, new EndpointAddress(remoteAddress), configurationPath)
        {
        }

        public CustomClientChannel(string endpointConfigurationName, EndpointAddress endpointAddress, string configurationPath)
            : base(typeof(T))
        {
            _ConfigurationPath = configurationPath;

            _EndpointConfigurationName = endpointConfigurationName;

            InitializeEndpoint(endpointConfigurationName, endpointAddress);

            SetupClientCredentials();
        }


        protected override ServiceEndpoint CreateDescription()
        {
            ServiceModelSectionGroup group;

            AppDomain.CurrentDomain.AssemblyResolve += ConfigResolveEventHandler;
            var serviceEndpoint = base.CreateDescription();

            if (_EndpointConfigurationName != null)
                serviceEndpoint.Name = _EndpointConfigurationName;

            if (!_sectionDictionary.ContainsKey(_ConfigurationPath))
            {
                if (!File.Exists(_ConfigurationPath))
                    throw new Exception(string.Format("Configuration file {0} was not found.", _ConfigurationPath));

                var map = new ExeConfigurationFileMap { ExeConfigFilename = _ConfigurationPath };

                var config = ConfigurationManager.OpenMappedExeConfiguration(map, ConfigurationUserLevel.None);

                group = ServiceModelSectionGroup.GetSectionGroup(config);

                _sectionDictionary.TryAdd(_ConfigurationPath, group);
            }

            bool hasValue = _sectionDictionary.TryGetValue(_ConfigurationPath, out group);

            if (@group == null || !hasValue)
                return serviceEndpoint;

            var selectedEndpoint = @group.Client.Endpoints.Cast<ChannelEndpointElement>().FirstOrDefault(endpoint =>
                endpoint.Contract == serviceEndpoint.Contract.ConfigurationName &&
                (_EndpointConfigurationName == null || _EndpointConfigurationName == endpoint.Name));

            if (selectedEndpoint == null)
                return serviceEndpoint;

            if (serviceEndpoint.Binding == null)
            {
                serviceEndpoint.Binding = CreateBinding(selectedEndpoint.Binding,
                    selectedEndpoint.BindingConfiguration,
                    @group);
            }

            if (serviceEndpoint.Address == null)
            {
                serviceEndpoint.Address = new EndpointAddress(selectedEndpoint.Address,
                    GetIdentity(selectedEndpoint.Identity),
                    selectedEndpoint.Headers.Headers);
            }

            if (serviceEndpoint.Behaviors.Count == 0 && selectedEndpoint.BehaviorConfiguration != null)
            {
                AddBehaviors(selectedEndpoint.BehaviorConfiguration, serviceEndpoint, @group);
            }

            serviceEndpoint.Name = selectedEndpoint.Contract;

            AppDomain.CurrentDomain.AssemblyResolve -= ConfigResolveEventHandler;

            return serviceEndpoint;
        }

        private Assembly ConfigResolveEventHandler(object sender, ResolveEventArgs args)
        {
            return Assembly.GetExecutingAssembly();
        }

        protected virtual void SetupClientCredentials()
        {
        }

        private Binding CreateBinding(string bindingName,
            string bindingConfigurationName,
            ServiceModelSectionGroup group)
        {
            var bindingElementCollection = group.Bindings[bindingName];

            if (bindingElementCollection.ConfiguredBindings.Count <= 0)
                return null;

            var selectedBinding = (from d in bindingElementCollection.ConfiguredBindings
                                   where d.Name == bindingConfigurationName
                                   select d).FirstOrDefault();

            var be = selectedBinding ?? bindingElementCollection.ConfiguredBindings[0];

            var binding = GetBinding(be);

            if (be != null)
            {
                be.ApplyConfiguration(binding);
            }

            return binding;
        }

        private Binding GetBinding(IBindingConfigurationElement configurationElement)
        {
            if (configurationElement is CustomBindingElement)
                return new CustomBinding();
            if (configurationElement is BasicHttpBindingElement)
                return new BasicHttpBinding();
            if (configurationElement is NetMsmqBindingElement)
                return new NetMsmqBinding();
            if (configurationElement is NetNamedPipeBindingElement)
                return new NetNamedPipeBinding();
#pragma warning disable 618
            if (configurationElement is NetPeerTcpBindingElement)
#pragma warning restore 618
#pragma warning disable 618
                return new NetPeerTcpBinding();
#pragma warning restore 618
            if (configurationElement is NetTcpBindingElement)
                return new NetTcpBinding();
            if (configurationElement is WSDualHttpBindingElement)
                return new WSDualHttpBinding();
            if (configurationElement is WSHttpBindingElement)
                return new WSHttpBinding();
            if (configurationElement is WSFederationHttpBindingElement)
                return new WSFederationHttpBinding();

            return null;
        }

        private void AddBehaviors(string behaviorConfiguration, ServiceEndpoint serviceEndpoint, ServiceModelSectionGroup group)
        {
            AddCustomClientBehavior(serviceEndpoint);

            if (string.IsNullOrEmpty(behaviorConfiguration))
                return;

            var behaviorElement = group.Behaviors.EndpointBehaviors[behaviorConfiguration];

            foreach (var extension in behaviorElement.Select(behaviorExtension =>
                behaviorExtension.GetType().InvokeMember("CreateBehavior",
                BindingFlags.InvokeMethod | BindingFlags.NonPublic | BindingFlags.Instance,
                null, behaviorExtension, null)).Where(extension => extension != null))
            {
                serviceEndpoint.Behaviors.Add((IEndpointBehavior)extension);
            }
        }

        private void AddCustomClientBehavior(ServiceEndpoint serviceEndpoint)
        {
            //Add CustomClientBehavior to Behaviors
            //Gives Opportunitiy to HACK SOAP Send/Receive. (Read - VA B/S)

            var customClientBehavior = new CustomClientBehavior();

            customClientBehavior.OnAfterReceiveReply += OnAfterReceiveReplyHandler;
            customClientBehavior.OnBeforeSendRequest += OnBeforeSendRequestHandler;

            serviceEndpoint.Behaviors.Add(customClientBehavior);
        }

        protected void OnAfterReceiveReplyHandler(ref Message reply, object correlationState)
        {
            if (OnAfterReceiveReply != null)
                OnAfterReceiveReply(ref reply, correlationState);
        }

        protected object OnBeforeSendRequestHandler(ref Message request, IClientChannel channel)
        {
            return OnBeforeSendRequest != null ? OnBeforeSendRequest(ref request, channel) : null;
        }

        private EndpointIdentity GetIdentity(IdentityElement element)
        {
            var properties = element.ElementInformation.Properties;

            var userPrincipalName = properties["userPrincipalName"];

            if (userPrincipalName != null && userPrincipalName.ValueOrigin != PropertyValueOrigin.Default)
            {
                return EndpointIdentity.CreateUpnIdentity(element.UserPrincipalName.Value);
            }

            var servicePrincipalName = properties["servicePrincipalName"];

            if (servicePrincipalName != null && servicePrincipalName.ValueOrigin != PropertyValueOrigin.Default)
            {
                return EndpointIdentity.CreateSpnIdentity(element.ServicePrincipalName.Value);
            }

            var dns = properties["dns"];

            if (dns != null && dns.ValueOrigin != PropertyValueOrigin.Default)
            {
                return EndpointIdentity.CreateDnsIdentity(element.Dns.Value);
            }

            var rsa = properties["rsa"];

            if (rsa != null && rsa.ValueOrigin != PropertyValueOrigin.Default)
            {
                return EndpointIdentity.CreateRsaIdentity(element.Rsa.Value);
            }

            var certificate = properties["certificate"];

            if (certificate == null || certificate.ValueOrigin == PropertyValueOrigin.Default)
                return null;

            var supportingCertificates = new X509Certificate2Collection();

            supportingCertificates.Import(Convert.FromBase64String(element.Certificate.EncodedValue));

            if (supportingCertificates.Count == 0)
            {
                throw new InvalidOperationException("UnableToLoadCertificateIdentity");
            }

            var primaryCertificate = supportingCertificates[0];

            supportingCertificates.RemoveAt(0);

            return EndpointIdentity.CreateX509CertificateIdentity(primaryCertificate, supportingCertificates);
        }

        protected override void ApplyConfiguration(string configurationName)
        {
        }

        public OnBeforeSendRequestDelegate OnBeforeSendRequest;
        public OnAfterReceiveReplyDelegate OnAfterReceiveReply;
    }
}

