﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Net;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Linq;
using VEIS.Core.Core;

namespace VEIS.Core.Wcf
{
    public class SoapLog
    {
        private static volatile SoapLog _current;
        private static readonly object Lock = new object();
        private static volatile Dictionary<int, DateLogValuePair> _soapLogDictionary = new Dictionary<int, DateLogValuePair>();
        private static DateTime _lastCleanup = DateTime.Now;
        private static Timer _cleanupTimer;
        private static long _logCleanupPeriodMilliSeconds;
        private static bool _dumpstats = false;

        public static string GetFormattedXml(XmlDocument xmlDocument)
        {
            using (var stringWriter = new StringWriter())
            {
                using (var xmlTextWriter = XmlWriter.Create(stringWriter))
                {
                    xmlDocument.WriteTo(xmlTextWriter);

                    xmlTextWriter.Flush();

                    return XElement.Parse(stringWriter.GetStringBuilder().ToString()).ToString();
                }
            }
        }

        public SoapLog()
        {
            bool canParseConfig = long.TryParse(ConfigurationManager.AppSettings.Get("LogCleanupPeriodMilliSeconds"), out _logCleanupPeriodMilliSeconds);
            Logger.Instance.Debug(string.Format("LogCleanupPeriodMilliseconds {0}", _logCleanupPeriodMilliSeconds));

            if (canParseConfig)
            {
                if (_logCleanupPeriodMilliSeconds > 0)
                {
                    AutoResetEvent autoResetEvent = new AutoResetEvent(false);
                    _cleanupTimer = new Timer(new TimerCallback(TryCleanupOldLog), autoResetEvent,
                        _logCleanupPeriodMilliSeconds,
                        _logCleanupPeriodMilliSeconds);

                    if (ConfigurationManager.AppSettings.Get("LogDumpStats") != null)
                    {
                        bool tDumpStats = false;
                        if (bool.TryParse(ConfigurationManager.AppSettings.Get("LogDumpStats"), out tDumpStats))
                        {
                            _dumpstats = tDumpStats;
                        }
                    }

                }

            }

        }

        public static SoapLog Current
        {
            get
            {
                if (_current != null)
                    return _current;

                lock (Lock)
                {
                    if (_current == null)
                    {
                        _current = new SoapLog();
                    }
                }

                return _current;
            }
        }

        public void LogMessage(LogMessageType logMessageType, Uri uri, ref Message request)
        {
            lock (Lock)
            {
                var soapLogDictionary = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, true);
                soapLogDictionary.LogDate = DateTime.Now;
                AppendHeader(logMessageType, uri, soapLogDictionary.Log);
                if(logMessageType == LogMessageType.Request)
                    AppendBody(ref request, soapLogDictionary.Log,soapLogDictionary.Request);
                else
                    AppendBody(ref request, soapLogDictionary.Log, soapLogDictionary.Response);
            }
        }

        public static string GetSoapLogEntry(int id)
        {
            DateLogValuePair retVal = null;
            if (_soapLogDictionary.ContainsKey(id))
            {
                retVal = _soapLogDictionary[id]; //.SingleOrDefault(v => v.Key == id).Value.To<DateLogValuePair>(); //.To<DateLogValuePair>();
            }
            return retVal != null ? retVal.Log.ToString() : string.Empty;
        }

        public void ClearLog()
        {
            lock (Lock)
            {
                if (_soapLogDictionary.ContainsKey(Thread.CurrentThread.ManagedThreadId))
                {
                    _soapLogDictionary.Remove(Thread.CurrentThread.ManagedThreadId);
                }
            }
        }

        public void WriteToFile(string fileName)
        {
            lock (Lock)
            {
                try
                {
                    File.WriteAllText(fileName, Log);
                }
                catch
                {
                    // ignored
                }
            }
        }

        public bool Active
        {
            get
            {
                lock (Lock)
                {
                    var dv = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, true); //.SingleOrDefault(v => v.Key == Thread.CurrentThread.ManagedThreadId);
                    return dv.Active;
                }
            }
            set
            {
                lock (Lock)
                {
                    var dv = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, true); //.SingleOrDefault(v => v.Key == Thread.CurrentThread.ManagedThreadId);
                    dv.Active = value;
                }
            }
        }

        public string Request
        {
            get
            {
                lock (Lock)
                {
                    var dv = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, false); //.SingleOrDefault(v => v.Key == Thread.CurrentThread.ManagedThreadId);
                    return dv != null && dv.Request != null ? dv.Request.ToString() : string.Empty;
                }
            }
        }
        public string Response
        {
            get
            {
                lock (Lock)
                {
                    var dv = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, false); //.SingleOrDefault(v => v.Key == Thread.CurrentThread.ManagedThreadId);
                    return dv != null && dv.Response != null ? dv.Response.ToString() : string.Empty;
                }
            }
        }
        public string Log
        {
            get
            {
                lock (Lock)
                {
                    var dv = GetDateValuePair(Thread.CurrentThread.ManagedThreadId, false); //.SingleOrDefault(v => v.Key == Thread.CurrentThread.ManagedThreadId);
                    return dv != null && dv.Log != null ? dv.Log.ToString() : string.Empty;
                }
            }
        }

        [Obsolete]
        public string AllLogs
        {
            get
            {
                lock (Lock)
                {
                    var soapLogDictionary = new StringBuilder();

                    foreach (var log in _soapLogDictionary)
                    {
                        soapLogDictionary.Append(log.Value.Log);
                    }

                    return soapLogDictionary.ToString();
                }
            }
        }

        public static void DumpStats()
        {
            int count = _soapLogDictionary.Count;
            Logger.Instance.Debug(string.Format("SoapLog Stats: Log Count: {0}", count));

            lock (Lock)
            {
                if (count > 0)
                {
                    DateTime minDate = _soapLogDictionary.Select(d => d.Value.LogDate).Min();
                    Logger.Instance.Debug(string.Format("SoapLog Stats: Lowest Date: {0}", minDate));

                    foreach (var s in _soapLogDictionary)
                    {
                        Logger.Instance.Debug(string.Format("SoapLog Stats: ThreadId: {0} : LogDate: {1} : Log Size: {2}", s.Key, s.Value.LogDate, s.Value.Log.ToString().Length));
                    }
                }
            }
        }

        private static void TryCleanupOldLog(object obj)
        {
            try
            {
                Logger.Instance.Debug("Top of TryCleanupOldLog");

                if (_dumpstats) DumpStats();


                lock (Lock)
                {
                    //var oldLogs = _soapLogDictionary.Where(v => v.Value.LogDate < DateTime.Now.AddMilliseconds(logCleanupPeriodMilliSeconds * -1));
                    var oldLogs = _soapLogDictionary.Where(v => v.Value.LogDate < _lastCleanup).ToList();

                    foreach (var log in oldLogs)
                    {
                        Logger.Instance.Debug(string.Format("Cleaningup ThreadId {0}", log.Key));
                        _soapLogDictionary.Remove(log.Key);
                    }
                    _lastCleanup = DateTime.Now;

                }

                Logger.Instance.Debug("End of TryCleanupOldLog");
                if (_dumpstats) DumpStats();
            }
            catch (Exception ex)
            {
                //Log, but eat the exception as this should not cause an exception in processing
                Logger.Instance.Error(ex);
            }

        }

        private static void AppendBody( ref Message request, StringBuilder sb, StringBuilder sb2)
        {
            var buffer = request.CreateBufferedCopy(int.MaxValue);

            var msg = buffer.CreateMessage();

            var xmlDocument = msg.ToXmlDocument();

            var formattedXml = GetFormattedXml(xmlDocument);
            sb.Append(formattedXml);
            sb.AppendLine("");
            request = buffer.CreateMessage();
            sb2.Append(formattedXml);
        }

        private static void AppendHeader(LogMessageType logMessageType, Uri uri, StringBuilder sb)
        {
            sb.AppendLine();

            var messageType = logMessageType == LogMessageType.Request
                ? "REQUEST" : "RESPONSE";

            var url = uri != null ? uri.OriginalString : "";

            sb.AppendLine(string.Format("<!-- [{0}] Target URI - {1} -->",
                messageType,
                url));
        }

        private static DateLogValuePair GetDateValuePair(int key, bool createIfNotExist)
        {
            DateLogValuePair dv = null;

            lock (Lock)
            {
                if (!_soapLogDictionary.ContainsKey(key))
                {
                    if (createIfNotExist)
                    {
                        dv = new DateLogValuePair() { LogDate = DateTime.Now, Log = new StringBuilder() , Request = new StringBuilder(), Response=new StringBuilder()};
                        _soapLogDictionary.Add(key, dv);
                    }
                }
                else
                {
                    dv = _soapLogDictionary[key];
                }
            }
            return dv;
        }


    }


    public class DateLogValuePair
    {
        public DateTime LogDate { get; set; }

        public StringBuilder Log { get; set; }

        public bool Active { get; set; }

        public StringBuilder Request { get; set; }

        public StringBuilder Response { get; set; }
    }

    public enum LogMessageType
    {
        Request,
        Response
    }
}
