﻿using System;
using System.Globalization;
using System.Threading.Tasks;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using VEIS.Security.Interfaces;
using VEIS.Security.Entities;
//using Microsoft.Azure.Services.AppAuthentication;
using Microsoft.Azure.KeyVault;
using System.Diagnostics;
using Microsoft.Azure.Services.AppAuthentication;

namespace VEIS.Security
{
    public class AzureAuthenticationHelper : IAzureAuthenticationHelper
    {
        /// <summary>
        /// The method for getting the authorization token from Azure AD
        /// </summary>
        /// <param name="clientId">The client ID of the AD application.</param>
        /// <param name="clientSecret">The client secret of the AD application</param>
        /// <param name="apiResourceId">The resource id of the API.</param>
        /// <param name="tenant">The Azure AD tenant</param>
        /// <param name="aadInstance">The Azure AD instance</param>
        /// <returns>
        /// Azure authorization result
        /// </returns>
        public async Task<AzureAuthResult> ObtainAuthTokenAsync(string clientId, string clientSecret, string apiResourceId, string tenant, string aadInstance)
        {
            var azureAuthResult = new AzureAuthResult();

            if (string.IsNullOrEmpty(tenant) || string.IsNullOrEmpty(aadInstance) || string.IsNullOrEmpty(apiResourceId) || string.IsNullOrEmpty(clientId) || string.IsNullOrEmpty(clientSecret))
            {
                throw new ArgumentException(
                    string.Format(
                        "{0}, {1}, {2}, {3}, {4} must be present in configuration.",
                        tenant,
                        aadInstance,
                        apiResourceId,
                        clientId,
                        clientSecret));
            }

            if (!aadInstance.Contains("{0}"))
            {
                throw new ArgumentException("AAD instance configuration should have a placeholder for tenant");
            }

            var authority = string.Format(CultureInfo.InvariantCulture, aadInstance, tenant);
            var authContext = new AuthenticationContext(authority);
            var cred = new ClientCredential(clientId, clientSecret);
            try
            {
                var authResult = await authContext.AcquireTokenAsync(apiResourceId, cred).ConfigureAwait(false);
                azureAuthResult.AuthToken = authResult.AccessToken;
                azureAuthResult.ExpiresOn = authResult.ExpiresOn;
            }
            catch(Exception ex)
            {
                //Logger.("Failed to obtain OAuth token");
                Trace.WriteLine(ex.Message);
                return null;
            }

            return azureAuthResult;
        }

        public async Task<string> ObtainSecretFromKeyVault(string KVSecretURL)
        {
            AzureServiceTokenProvider azureServiceTokenProvider = new AzureServiceTokenProvider();

            try
            {
                var keyVaultClient = new KeyVaultClient(
                    new KeyVaultClient.AuthenticationCallback(azureServiceTokenProvider.KeyVaultTokenCallback));

                var secret = await keyVaultClient.GetSecretAsync(KVSecretURL)// "https://veiskeyvault.vault.azure.net/secrets/SchoolSolnECAPI/3b9ceaf53abe456aa293f686cc255a9e")
                    .ConfigureAwait(false);

                return secret.Value;
            }
            catch (Exception ex)
            {
                Trace.WriteLine("Failed to obtain Secret from Key Vault");
                Trace.WriteLine(ex.Message);
                return null;
            } 
        }
    }
}
